/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import com.linuxense.javadbf.DBFException;
import com.linuxense.javadbf.DBFReader;
import java.awt.Image;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Logger;
import nl.cloudfarming.client.geoviewer.BaseLayer;
import nl.cloudfarming.client.geoviewer.SimpleLayer;
import nl.cloudfarming.client.sensor.SensorDataObject;
import nl.cloudfarming.client.sensor.SensorFileNode;
import nl.cloudfarming.client.sensor.SensorProcessor;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiFileLoader;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 * GreenSeeker DataObject. Tries to detect v/d Borne and Normal file processors
 * based on the number of fields in the sensor file.
 *
 * @author Arjan Loermans
 */
@Messages({
    "# Fileprocessor", 
    "sensor.greenseeker.fileprocessor.borne.log.displayname=Greenseeker Borne", 
    "sensor.greenseeker.fileprocessor.normal.log.displayname=Greenseeker Normal", 
    "# Processorprovider", 
    "sensor.greenseeker.processorprovider.filefilter.description=Greenseeker data {0}", 
    "# Logerror", 
    "sensor.greenseeker.fileprocessor.logerror.failed_to_create_time=Could not convert {0} to time.",
    "GreenseekerDataObject.logerror.failed_to_read_inputstream=Could not read inputstream from {0}.",
    "GreenseekerDataObject.logerror.failed_to_read_dbf=Could not read {0} with the DBF-reader.",
})
public class GreenseekerDataObject extends SensorDataObject {

    private GreenseekerFileProcessor processor = null;
    int gsType = 0;
    private static final Logger LOG = Logger.getLogger("nl.cloudfarming.client.sensor.greenseeker.GreenseekerDataObject");
    public static final String MIME_TYPE = "application/x-agrosense-greenseeker";
    public static final String ICON_BASE = "nl/cloudfarming/client/sensor/greenseeker/greenseeker-file-icon.png";
    
    public GreenseekerDataObject(FileObject pf, MultiFileLoader loader) throws DataObjectExistsException {
        super(pf, loader);
        registerEditor(MIME_TYPE, true);
        this.gsType = fieldCount(pf);
    }
    
     @Override
    protected Node createNodeDelegate() {
        return new GreenseekerNode(super.createNodeDelegate());
    }

    @Override
    protected int associateLookup() {
        return 1;
    }

    @Override
    public HelpCtx getHelpCtx() {
        return new HelpCtx("nl.cloudfarming.client.sensor.greenseeker.about");
    }

    private int fieldCount(FileObject fo) {
        int count = 0;
        if(FileUtil.toFile(fo) != null){ // Template file does not actually exist yet.
            try(InputStream inputStream = fo.getInputStream()){
                DBFReader reader = new DBFReader(inputStream);
                count = reader.getFieldCount();
            } catch (DBFException ex) {
                LOG.warning(NbBundle.getMessage(GreenseekerDataObject.class, "GreenseekerDataObject.logerror.failed_to_read_dbf", fo.getName()));
                Exceptions.printStackTrace(ex);
            } catch (IOException ex) {
                LOG.warning(NbBundle.getMessage(GreenseekerDataObject.class, "GreenseekerDataObject.logerror.failed_to_read_inputstream", fo.getName()));
                Exceptions.printStackTrace(ex);
            }
        }
        return count;
    }

    @Override
    protected SensorProcessor getProcessor() {
        if (this.processor == null) {
            // Check the number of fields in the file, Borne has 20 fields, Normal has 14. Set the corresponding Processor
            switch (this.gsType) {
                case GreenseekerFileProcessorBorne.nrOfFields:
                    processor = new GreenseekerFileProcessorBorne(this);
                    break;
                case GreenseekerFileProcessorNormal.nrOfFields:
                    processor = new GreenseekerFileProcessorNormal(this);
                    break;
                default:
                    processor = null;
                    break;
            }
        }
        return this.processor;
    }
    
    /**
     * Node for the Greenseeker
     */
    private class GreenseekerNode extends SensorFileNode {

        public GreenseekerNode(Node original) {
            super(original, GreenseekerDataObject.this, new BaseLayer(getPalette() ,original.getDisplayName()));
        }

        @Override
        public Image getIcon(int type) {
            return ImageUtilities.loadImage(ICON_BASE);
        }
    }
}
